
From: john stultz <johnstul@us.ibm.com>

This is a rework of John's recent change which resolved a circular include
dependency: a function in mach_apic.h requires hard_smp_processor_id() and
hard_smp_processor_id() requires macros from mach_apic.h

So this patch (against bk-current) reverts the previous, and fixes the same
circular dependency in a much cleaner way, by moving a piece of the circular
chain into its own .h file, rather then removing hard_smp_processor_id() and
accessing the apic by hand.


 25-akpm/arch/i386/mach-generic/bigsmp.c              |    1 +
 25-akpm/arch/i386/mach-generic/default.c             |    1 +
 25-akpm/arch/i386/mach-generic/summit.c              |    1 +
 25-akpm/include/asm-i386/mach-bigsmp/mach_apic.h     |   13 ++-----------
 25-akpm/include/asm-i386/mach-bigsmp/mach_apicdef.h  |   13 +++++++++++++
 25-akpm/include/asm-i386/mach-default/mach_apic.h    |   11 ++---------
 25-akpm/include/asm-i386/mach-default/mach_apicdef.h |   13 +++++++++++++
 25-akpm/include/asm-i386/mach-numaq/mach_apic.h      |    9 ---------
 25-akpm/include/asm-i386/mach-numaq/mach_apicdef.h   |   14 ++++++++++++++
 25-akpm/include/asm-i386/mach-summit/mach_apic.h     |   13 ++-----------
 25-akpm/include/asm-i386/mach-summit/mach_apicdef.h  |   13 +++++++++++++
 25-akpm/include/asm-i386/mach-visws/mach_apic.h      |    8 --------
 25-akpm/include/asm-i386/mach-visws/mach_apicdef.h   |   12 ++++++++++++
 25-akpm/include/asm-i386/smp.h                       |    2 +-
 14 files changed, 75 insertions(+), 49 deletions(-)

diff -puN arch/i386/mach-generic/bigsmp.c~mystery-subarch-fix arch/i386/mach-generic/bigsmp.c
--- 25/arch/i386/mach-generic/bigsmp.c~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/arch/i386/mach-generic/bigsmp.c	Fri May 23 13:15:34 2003
@@ -10,6 +10,7 @@
 #include <linux/smp.h>
 #include <linux/init.h>
 #include <asm/mach-bigsmp/mach_apic.h>
+#include <asm/mach-bigsmp/mach_apicdef.h>
 #include <asm/mach-bigsmp/mach_ipi.h>
 #include <asm/mach-default/mach_mpparse.h>
 
diff -puN arch/i386/mach-generic/default.c~mystery-subarch-fix arch/i386/mach-generic/default.c
--- 25/arch/i386/mach-generic/default.c~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/arch/i386/mach-generic/default.c	Fri May 23 13:15:34 2003
@@ -10,6 +10,7 @@
 #include <linux/smp.h>
 #include <linux/init.h>
 #include <asm/mach-default/mach_apic.h>
+#include <asm/mach-default/mach_apicdef.h>
 #include <asm/mach-default/mach_ipi.h>
 #include <asm/mach-default/mach_mpparse.h>
 
diff -puN arch/i386/mach-generic/summit.c~mystery-subarch-fix arch/i386/mach-generic/summit.c
--- 25/arch/i386/mach-generic/summit.c~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/arch/i386/mach-generic/summit.c	Fri May 23 13:15:34 2003
@@ -10,6 +10,7 @@
 #include <linux/smp.h>
 #include <linux/init.h>
 #include <asm/mach-summit/mach_apic.h>
+#include <asm/mach-summit/mach_apicdef.h>
 #include <asm/mach-summit/mach_ipi.h>
 #include <asm/mach-summit/mach_mpparse.h>
 
diff -puN /dev/null include/asm-i386/mach-bigsmp/mach_apicdef.h
--- /dev/null	Thu Apr 11 07:25:15 2002
+++ 25-akpm/include/asm-i386/mach-bigsmp/mach_apicdef.h	Fri May 23 13:15:34 2003
@@ -0,0 +1,13 @@
+#ifndef __ASM_MACH_APICDEF_H
+#define __ASM_MACH_APICDEF_H
+
+#define		APIC_ID_MASK		(0x0F<<24)
+
+static inline unsigned get_apic_id(unsigned long x) 
+{ 
+	return (((x)>>24)&0x0F);
+} 
+
+#define		GET_APIC_ID(x)	get_apic_id(x)
+
+#endif
diff -puN include/asm-i386/mach-bigsmp/mach_apic.h~mystery-subarch-fix include/asm-i386/mach-bigsmp/mach_apic.h
--- 25/include/asm-i386/mach-bigsmp/mach_apic.h~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/include/asm-i386/mach-bigsmp/mach_apic.h	Fri May 23 13:15:34 2003
@@ -1,5 +1,6 @@
 #ifndef __ASM_MACH_APIC_H
 #define __ASM_MACH_APIC_H
+#include <asm/smp.h>
 
 #define SEQUENTIAL_APICID
 #ifdef SEQUENTIAL_APICID
@@ -40,18 +41,10 @@ static inline unsigned long check_apicid
 
 #define apicid_cluster(apicid) (apicid & 0xF0)
 
-static inline unsigned get_apic_id(unsigned long x) 
-{ 
-	return (((x)>>24)&0x0F);
-} 
-
-#define		GET_APIC_ID(x)	get_apic_id(x)
-
 static inline unsigned long calculate_ldr(unsigned long old)
 {
 	unsigned long id;
-	id = xapic_phys_to_log_apicid(
-			GET_APIC_LOGICAL_ID(*(unsigned long *)(APIC_BASE+APIC_LDR)));
+	id = xapic_phys_to_log_apicid(hard_smp_processor_id());
 	return ((old & ~APIC_LDR_MASK) | SET_APIC_LOGICAL_ID(id));
 }
 
@@ -135,8 +128,6 @@ static inline int check_phys_apicid_pres
 	return (1);
 }
 
-#define		APIC_ID_MASK		(0x0F<<24)
-
 static inline unsigned int cpu_mask_to_apicid (unsigned long cpumask)
 {
 	int num_bits_set;
diff -puN /dev/null include/asm-i386/mach-default/mach_apicdef.h
--- /dev/null	Thu Apr 11 07:25:15 2002
+++ 25-akpm/include/asm-i386/mach-default/mach_apicdef.h	Fri May 23 13:15:34 2003
@@ -0,0 +1,13 @@
+#ifndef __ASM_MACH_APICDEF_H
+#define __ASM_MACH_APICDEF_H
+
+#define		APIC_ID_MASK		(0xF<<24)
+
+static inline unsigned get_apic_id(unsigned long x) 
+{ 
+	return (((x)>>24)&0xF);
+} 
+
+#define		GET_APIC_ID(x)	get_apic_id(x)
+
+#endif
diff -puN include/asm-i386/mach-default/mach_apic.h~mystery-subarch-fix include/asm-i386/mach-default/mach_apic.h
--- 25/include/asm-i386/mach-default/mach_apic.h~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/include/asm-i386/mach-default/mach_apic.h	Fri May 23 13:15:34 2003
@@ -1,6 +1,8 @@
 #ifndef __ASM_MACH_APIC_H
 #define __ASM_MACH_APIC_H
 
+#include <mach_apicdef.h>
+
 #define APIC_DFR_VALUE	(APIC_DFR_FLAT)
 
 static inline unsigned long target_cpus(void)
@@ -105,15 +107,6 @@ static inline int check_phys_apicid_pres
 	return test_bit(boot_cpu_physical_apicid, &phys_cpu_present_map);
 }
 
-#define		APIC_ID_MASK		(0xF<<24)
-
-static inline unsigned get_apic_id(unsigned long x) 
-{ 
-	return (((x)>>24)&0xF);
-} 
-
-#define		GET_APIC_ID(x)	get_apic_id(x)
-
 static inline int apic_id_registered(void)
 {
 	return (test_bit(GET_APIC_ID(apic_read(APIC_ID)), 
diff -puN /dev/null include/asm-i386/mach-numaq/mach_apicdef.h
--- /dev/null	Thu Apr 11 07:25:15 2002
+++ 25-akpm/include/asm-i386/mach-numaq/mach_apicdef.h	Fri May 23 13:15:34 2003
@@ -0,0 +1,14 @@
+#ifndef __ASM_MACH_APICDEF_H
+#define __ASM_MACH_APICDEF_H
+
+
+#define APIC_ID_MASK (0xF<<24)
+
+static inline unsigned get_apic_id(unsigned long x)
+{
+	        return (((x)>>24)&0x0F);
+}
+
+#define         GET_APIC_ID(x)  get_apic_id(x)
+
+#endif
diff -puN include/asm-i386/mach-numaq/mach_apic.h~mystery-subarch-fix include/asm-i386/mach-numaq/mach_apic.h
--- 25/include/asm-i386/mach-numaq/mach_apic.h~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/include/asm-i386/mach-numaq/mach_apic.h	Fri May 23 13:15:34 2003
@@ -107,15 +107,6 @@ static inline int check_phys_apicid_pres
 	return (1);
 }
 
-#define APIC_ID_MASK (0xF<<24)
-
-static inline unsigned get_apic_id(unsigned long x)
-{
-	        return (((x)>>24)&0x0F);
-}
-
-#define         GET_APIC_ID(x)  get_apic_id(x)
-
 static inline unsigned int cpu_mask_to_apicid (unsigned long cpumask)
 {
 	int num_bits_set;
diff -puN /dev/null include/asm-i386/mach-summit/mach_apicdef.h
--- /dev/null	Thu Apr 11 07:25:15 2002
+++ 25-akpm/include/asm-i386/mach-summit/mach_apicdef.h	Fri May 23 13:15:34 2003
@@ -0,0 +1,13 @@
+#ifndef __ASM_MACH_APICDEF_H
+#define __ASM_MACH_APICDEF_H
+
+#define		APIC_ID_MASK		(0xFF<<24)
+
+static inline unsigned get_apic_id(unsigned long x) 
+{ 
+	return (((x)>>24)&0xFF);
+} 
+
+#define		GET_APIC_ID(x)	get_apic_id(x)
+
+#endif
diff -puN include/asm-i386/mach-summit/mach_apic.h~mystery-subarch-fix include/asm-i386/mach-summit/mach_apic.h
--- 25/include/asm-i386/mach-summit/mach_apic.h~mystery-subarch-fix	Fri May 23 13:15:33 2003
+++ 25-akpm/include/asm-i386/mach-summit/mach_apic.h	Fri May 23 13:15:34 2003
@@ -2,6 +2,7 @@
 #define __ASM_MACH_APIC_H
 
 #include <linux/config.h>
+#include <asm/smp.h>
 
 #ifdef CONFIG_X86_GENERICARCH
 #define x86_summit 1	/* must be an constant expressiona for generic arch */
@@ -48,20 +49,12 @@ static inline unsigned long check_apicid
 
 extern u8 bios_cpu_apicid[];
 
-static inline unsigned get_apic_id(unsigned long x) 
-{ 
-	return (((x)>>24)&0xFF);
-} 
-
-#define		GET_APIC_ID(x)	get_apic_id(x)
-
 static inline void init_apic_ldr(void)
 {
 	unsigned long val, id;
 
 	if (x86_summit)
-		id = xapic_phys_to_log_apicid(
-			GET_APIC_ID(*(unsigned long *)(APIC_BASE+APIC_ID)));
+		id = xapic_phys_to_log_apicid(hard_smp_processor_id());
 	else
 		id = 1UL << smp_processor_id();
 	apic_write_around(APIC_DFR, APIC_DFR_VALUE);
@@ -143,8 +136,6 @@ static inline int check_phys_apicid_pres
 		return test_bit(boot_cpu_physical_apicid, &phys_cpu_present_map);
 }
 
-#define		APIC_ID_MASK		(0xFF<<24)
-
 static inline unsigned int cpu_mask_to_apicid (unsigned long cpumask)
 {
 	int num_bits_set;
diff -puN /dev/null include/asm-i386/mach-visws/mach_apicdef.h
--- /dev/null	Thu Apr 11 07:25:15 2002
+++ 25-akpm/include/asm-i386/mach-visws/mach_apicdef.h	Fri May 23 13:15:34 2003
@@ -0,0 +1,12 @@
+#ifndef __ASM_MACH_APICDEF_H
+#define __ASM_MACH_APICDEF_H
+
+#define         APIC_ID_MASK            (0xF<<24)
+
+static inline unsigned get_apic_id(unsigned long x)
+{
+	        return (((x)>>24)&0xF);
+}
+#define         GET_APIC_ID(x)  get_apic_id(x)
+
+#endif
diff -puN include/asm-i386/mach-visws/mach_apic.h~mystery-subarch-fix include/asm-i386/mach-visws/mach_apic.h
--- 25/include/asm-i386/mach-visws/mach_apic.h~mystery-subarch-fix	Fri May 23 13:15:34 2003
+++ 25-akpm/include/asm-i386/mach-visws/mach_apic.h	Fri May 23 13:15:34 2003
@@ -19,14 +19,6 @@
 #define check_apicid_used(bitmap, apicid) (bitmap & (1 << apicid))
 #define check_apicid_present(bit) (phys_cpu_present_map & (1 << bit))
 
-#define         APIC_ID_MASK            (0xF<<24)
-
-static inline unsigned get_apic_id(unsigned long x)
-{
-	        return (((x)>>24)&0xF);
-}
-#define         GET_APIC_ID(x)  get_apic_id(x)
-
 static inline int apic_id_registered(void)
 {
 	return (test_bit(GET_APIC_ID(apic_read(APIC_ID)), 
diff -puN include/asm-i386/smp.h~mystery-subarch-fix include/asm-i386/smp.h
--- 25/include/asm-i386/smp.h~mystery-subarch-fix	Fri May 23 13:15:34 2003
+++ 25-akpm/include/asm-i386/smp.h	Fri May 23 13:15:34 2003
@@ -91,7 +91,7 @@ extern inline int any_online_cpu(unsigne
 #ifdef APIC_DEFINITION
 extern int hard_smp_processor_id(void);
 #else
-#include <mach_apic.h>
+#include <mach_apicdef.h>
 static inline int hard_smp_processor_id(void)
 {
 	/* we don't want to mark this access volatile - bad code generation */

_
